/***************************************************************************
 *   Copyright (C) 2009 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kapplication.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include "patternlistviewitem.h"
#include "previewiconview.h"

/*
struct LIBRARY (original)
{
  Q_UINT32 patterns;
  [
    Q_UINT16 checksum
    QByteArray compressedDataStream
    [
      QString scheme
      Q_INT32 width
      Q_INT32 height
      [
        Q_INT8 count
        Q_INT8 type
        Q_UINT32 color
      ] height x width
      Q_INT32 backstitches
      [
        QPoint start
        QPoint end
        Q_INT32 color
      ] backstitches
      Q_INT32 knots
      [
        QPoint start
        Q_INT32 color
      ] knots
    ]
  ] patterns
]

struct LIBRARY (new)
[
  "KXStitchLib"
  Q_INT16 version
  [
    Q_UINT16 key
    Q_UINT16 modifier
    Q_INT16 baseline
    Q_UINT16 checksum
    QByteArray compressedDataStream
    [
      QString scheme
      Q_INT32 width
      Q_INT32 height
      [
        Q_INT8 count
        Q_INT8 type
        Q_UINT32 color
      ] height x width
      Q_INT32 backstitches
      [
        QPoint start
        QPoint end
        Q_INT32 color
      ] backstitches
      Q_INT32 knots
      [
        QPoint start
        Q_INT32 color
      ] knots
    ]
  ]
]
*/


PatternListViewItem::PatternListViewItem(QListView *parent, QString name)
  : KListViewItem(parent, name)
{
  setDropEnabled(true);
}

PatternListViewItem::PatternListViewItem(QListViewItem *parent, QString name)
  : KListViewItem(parent, name)
{
  setDropEnabled(true);
}

PatternListViewItem::~PatternListViewItem()
{
  for ( PatternFile *patternFile = m_patternFiles.first() ; patternFile ; patternFile = m_patternFiles.next() )
    delete patternFile;
}

int PatternListViewItem::maxHeight()
{
  int max = 0;
  for (LibraryPattern *pattern = first() ; pattern ; pattern = next())
    max = std::max(max, (int)pattern->height());
  return max;
}

LibraryPattern *PatternListViewItem::findCharacter(int key, Qt::ButtonState modifiers)
{
  LibraryPattern *pattern;
  for (pattern = first() ; pattern ; pattern = next())
  {
    if (pattern->key() == key && pattern->modifiers() == modifiers)
      break;
  }
  return pattern;
}

LibraryPattern *PatternListViewItem::first()
{
  LibraryPattern *pattern = 0;
  if (m_patternFiles.first())
    pattern = m_patternFiles.first()->first();   // get first pattern of the first file
  return pattern;
}

LibraryPattern *PatternListViewItem::next()
{
  LibraryPattern *pattern = 0;
  if (m_patternFiles.current())
  {
    pattern = m_patternFiles.current()->next();
    if (pattern == 0)    // reached the end of the current file
    {
      if (m_patternFiles.next())
      {
        pattern = m_patternFiles.current()->first();  // get the first pattern of the next file
      }
    }
  }
  return pattern;
}

void PatternListViewItem::addPath(QString path)
{
  m_patternFiles.append(new PatternFile(path));
}

QString PatternListViewItem::path()
{
  return m_patternFiles.first()->path();
}

QStringList PatternListViewItem::paths()
{
  QStringList paths;
  for (PatternFile *patternFile = m_patternFiles.first() ; patternFile ; patternFile = m_patternFiles.next())
  {
    paths.append(patternFile->localFile());
  }
  return paths;
}

PatternFile *PatternListViewItem::writablePath()
{
  PatternFile *file;
  for (file = m_patternFiles.first() ; file && ! file->isWritable() ; file = m_patternFiles.next()) ; // iterate file paths and break when a writable one is found
  if (file == 0)
  {
    // create a writable path
    QString path = m_patternFiles.first()->path();
    path.replace(0, path.find("library"), "");
    path = kapp->dirs()->saveLocation("appdata", path);
    addPath(path);
    file = m_patternFiles.last();
  }
  return file;
}

void PatternListViewItem::addPattern(KXStitchDragObject *pattern)
{
  writablePath()->addPattern(pattern);
}

void PatternListViewItem::addPattern(LibraryPattern *pattern)
{
  writablePath()->addPattern(pattern);
}

void PatternListViewItem::deletePattern(PreviewIconViewItem *item)
{
  for (PatternFile *file = m_patternFiles.first() ; file ; file = m_patternFiles.next())
  {
    for (LibraryPattern *pattern = file->first() ; pattern ; pattern = file->next())
    {
      if (pattern->iconViewItem() == item)
        file->deletePattern(pattern);
    }
  }
}
