/***************************************************************************
 *   Copyright (C) 2003 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qcheckbox.h>
#include <kcombobox.h>
#include <kdebug.h>
#include <knuminput.h>
#include <kcolorbutton.h>
#include <kapplication.h>
#include <klocale.h>
#include "kxstitch.h"
#include "patternpropertiesdialog.h"
#include "configuration.h"

PatternPropertiesDialog::PatternPropertiesDialog(QWidget *parent)
  : PatternPropertiesDlg(parent)
{
  reset();
  connect(SizeUnits, SIGNAL(activated(int)), this, SLOT(slotUnitsChanged(int)));
  connect(HelpButton, SIGNAL(clicked()), this, SLOT(slotContextHelp()));
}

void PatternPropertiesDialog::show()
{
  SizeUnits->setCurrentItem(m_sizeUnits);
  m_sizeDisplayUnits = m_sizeUnits;
  calculateSizes(m_sizeWidth, m_sizeHeight, m_clothCount, m_clothCountUnits);
  PatternTitle->setText(m_patternTitle);
  PatternAuthor->setText(m_patternAuthor);
  PatternCopyright->setText(m_patternCopyright);
  PatternFabric->setText(m_patternFabric);
  FabricColor->setColor(m_fabricColor);
  FlossScheme->clear();
  FlossScheme->insertStringList(static_cast<KXStitchApplication*>(kapp)->schemes());
  FlossScheme->setCurrentItem(m_flossScheme);
  PatternInstructions->setText(m_patternInstructions);
  SetDefault->setChecked(false);
  PatternPropertiesDlg::show();
}

void PatternPropertiesDialog::reset()
{
  m_sizeDisplayUnits = m_sizeUnits = Configuration::newPattern_Units();
  m_sizeWidth = Configuration::newPattern_Width();
  m_sizeHeight = Configuration::newPattern_Height();
  m_clothCount = Configuration::newPattern_ClothCount();
  m_clothCountUnits = Configuration::newPattern_ClothCountUnits();
  m_patternTitle = Configuration::newPattern_Title();
  m_patternAuthor = Configuration::newPattern_Author();
  m_patternCopyright = Configuration::newPattern_Copyright();
  m_patternFabric = Configuration::newPattern_Fabric();
  m_fabricColor = Configuration::newPattern_Color();
  m_flossScheme = Configuration::newPattern_FlossScheme();
  m_patternInstructions = Configuration::newPattern_Instructions();
}

void PatternPropertiesDialog::setUnits(int sizeUnits)
{
  m_sizeDisplayUnits = m_sizeUnits = sizeUnits;
}

void PatternPropertiesDialog::setPatternWidth(int width)
{
  m_sizeWidth = width;
}

void PatternPropertiesDialog::setPatternHeight(int height)
{
  m_sizeHeight = height;
}

void PatternPropertiesDialog::setPatternSize(int width, int height)
{
  m_sizeWidth = width;
  m_sizeHeight = height;
}

void PatternPropertiesDialog::setClothCount(double clothCount)
{
  m_clothCount = clothCount;
}

void PatternPropertiesDialog::setClothCountUnits(int clothCountUnits)
{
  m_clothCountUnits = clothCountUnits;
}

void PatternPropertiesDialog::setTitle(QString title)
{
  m_patternTitle = title;
}

void PatternPropertiesDialog::setAuthor(QString author)
{
  m_patternAuthor = author;
}

void PatternPropertiesDialog::setCopyright(QString copyright)
{
  m_patternCopyright = copyright;
}

void PatternPropertiesDialog::setFabric(QString fabric)
{
  m_patternFabric = fabric;
}

void PatternPropertiesDialog::setFabricColor(QColor fabricColor)
{
  m_fabricColor = fabricColor;
}

void PatternPropertiesDialog::setScheme(QString scheme)
{
  m_flossScheme = scheme;
}

void PatternPropertiesDialog::setInstructions(QString instructions)
{
  m_patternInstructions = instructions;
}

int PatternPropertiesDialog::units()
{
  return m_sizeUnits;
}

int PatternPropertiesDialog::patternWidth()
{
  return m_sizeWidth;
}

int PatternPropertiesDialog::patternHeight()
{
  return m_sizeHeight;
}

QSize PatternPropertiesDialog::patternSize()
{
  return QSize( m_sizeWidth, m_sizeHeight );
}

double PatternPropertiesDialog::clothCount()
{
  return m_clothCount;
}

int PatternPropertiesDialog::clothCountUnits()
{
  return m_clothCountUnits;
}

QString PatternPropertiesDialog::title()
{
  return m_patternTitle;
}

QString PatternPropertiesDialog::author()
{
  return m_patternAuthor;
}

QString PatternPropertiesDialog::copyright()
{
  return m_patternCopyright;
}

QString PatternPropertiesDialog::fabric()
{
  return m_patternFabric;
}

QColor PatternPropertiesDialog::fabricColor()
{
  return m_fabricColor;
}

QString PatternPropertiesDialog::scheme()
{
  return m_flossScheme;
}

QString PatternPropertiesDialog::instructions()
{
  return m_patternInstructions;
}

void PatternPropertiesDialog::accept()
{
  double w = SizeWidth->value();
  double h = SizeHeight->value();
  double cc = ClothCount->value();
  if (m_sizeDisplayUnits != Configuration::EnumNewPattern_Units::Stitches)
  {
    w *= cc;
    h *= cc;
  }
  m_sizeUnits = m_sizeDisplayUnits;
  m_sizeWidth = int(w);
  m_sizeHeight = int(h);
  m_clothCount = cc;
  m_clothCountUnits = (ClothCount->suffix() == "/in")?Configuration::EnumNewPattern_ClothCountUnits::Inches:Configuration::EnumNewPattern_ClothCountUnits::CM;
  m_patternTitle = PatternTitle->text();
  m_patternAuthor = PatternAuthor->text();
  m_patternCopyright = PatternCopyright->text();
  m_patternFabric = PatternFabric->text();
  m_fabricColor = FabricColor->color();
  m_flossScheme = FlossScheme->currentText();
  m_patternInstructions = PatternInstructions->text();
  if (SetDefault->isChecked())
  {
    Configuration::setNewPattern_Units(m_sizeUnits);
    Configuration::setNewPattern_Width(m_sizeWidth);
    Configuration::setNewPattern_Height(m_sizeHeight);
    Configuration::setNewPattern_ClothCount(m_clothCount);
    Configuration::setNewPattern_ClothCountUnits(m_clothCountUnits);
    Configuration::setNewPattern_Title(m_patternTitle);
    Configuration::setNewPattern_Author(m_patternAuthor);
    Configuration::setNewPattern_Copyright(m_patternCopyright);
    Configuration::setNewPattern_Fabric(m_patternFabric);
    Configuration::setNewPattern_Color(m_fabricColor);
    Configuration::setNewPattern_FlossScheme(m_flossScheme);
    Configuration::setNewPattern_Instructions(m_patternInstructions);
//    KXSConfig().writeSettings();
  }
  PatternPropertiesDlg::accept();
}

void PatternPropertiesDialog::slotContextHelp()
{
  kapp->invokeHelp("PatternPropertiesDialog");
}

void PatternPropertiesDialog::slotUnitsChanged(int sizeUnits)
{
  double  cc = ClothCount->value();
  int ccu = (ClothCount->suffix() == "/in")?Configuration::EnumNewPattern_ClothCountUnits::Inches:Configuration::EnumNewPattern_ClothCountUnits::CM;
  double w = SizeWidth->value();
  double h = SizeHeight->value();
  if (m_sizeDisplayUnits != Configuration::EnumNewPattern_Units::Stitches)
  {
    w *= cc;
    h *= cc;
  }
  if (sizeUnits == Configuration::EnumNewPattern_ClothCountUnits::CM)
  {
    if (ccu == Configuration::EnumNewPattern_ClothCountUnits::Inches)
    {
      ccu = Configuration::EnumNewPattern_ClothCountUnits::CM;
      cc /= 2.54;
    }
  }
  if (sizeUnits == Configuration::EnumNewPattern_ClothCountUnits::Inches)
  {
    if (ccu == Configuration::EnumNewPattern_ClothCountUnits::CM)
    {
      ccu = Configuration::EnumNewPattern_ClothCountUnits::Inches;
      cc = (double)(int)(cc*2.54);
    }
  }
  m_sizeDisplayUnits = sizeUnits;
  calculateSizes(int(w), int(h), cc, ccu);
}

void PatternPropertiesDialog::calculateSizes(int w, int h, double cc, int ccu)
{
  double  scale = cc;
  double  sizeStep = 0.01;
  if (SizeUnits->currentText() == i18n("Stitches"))
  {
    scale = 1.0;
    sizeStep = 1.0;
  }
  SizeWidth->setRange(1, 100000, sizeStep, false);
  SizeWidth->setValue(w/scale);
  SizeHeight->setRange(1, 100000, sizeStep, false);
  SizeHeight->setValue(h/scale);
  ClothCount->setRange(1, 55, (ccu == Configuration::EnumNewPattern_ClothCountUnits::CM)?0.01:1.0, false);
  ClothCount->setValue(cc);
  ClothCount->setSuffix((ccu == Configuration::EnumNewPattern_ClothCountUnits::CM)?"/cm":"/in");
}

// QDataStream & operator << ( QDataStream & stream, const PatternPropertiesDialog & properties )
bool PatternPropertiesDialog::writeKXStitchStream(QDataStream &stream)
{
  stream << Q_INT32(m_sizeUnits);
  stream << Q_INT32(m_sizeWidth);
  stream << Q_INT32(m_sizeHeight);
  stream << m_clothCount;
  stream << Q_INT32(m_clothCountUnits);
  stream << m_patternTitle;
  stream << m_patternAuthor;
  stream << m_patternCopyright;
  stream << m_patternFabric;
  stream << m_fabricColor;
  stream << m_flossScheme;
  stream << m_patternInstructions;
  return true;
}

//QDataStream & operator >> ( QDataStream & stream, PatternPropertiesDialog & properties )
bool PatternPropertiesDialog::readKXStitchStream(QDataStream &stream, int fileFormatVersion)
{
  Q_INT32 w;
  Q_INT32 h;
  Q_INT32 su;
  Q_INT32 ccu;
  QString tmp;
  switch (fileFormatVersion)
  {
    case 7:
      stream >> su;
      stream >> w;
      stream >> h;
      m_sizeUnits = su;
      m_sizeWidth = w;
      m_sizeHeight = h;
      stream >> m_clothCount;
      stream >> ccu;
      m_clothCountUnits = ccu;
      stream >> m_patternTitle;
      stream >> m_patternAuthor;
      stream >> m_patternCopyright;
      stream >> m_patternFabric;
      stream >> m_fabricColor;
      stream >> m_flossScheme;
      stream >> m_patternInstructions;
      break;
    case 6:
      stream >> tmp;
      if (tmp == "Stitches") m_sizeUnits = Configuration::EnumNewPattern_Units::Stitches;
      if (tmp == "CM") m_sizeUnits = Configuration::EnumNewPattern_Units::CM;
      if (tmp == "Inches") m_sizeUnits = Configuration::EnumNewPattern_Units::Inches;
      stream >> w;
      m_sizeWidth = w;
      stream >> h;
      m_sizeHeight = h;
      stream >> m_clothCount;
      stream >> tmp;
      if (tmp == "/in") m_clothCountUnits = Configuration::EnumNewPattern_ClothCountUnits::Inches;
      if (tmp == "/cm") m_clothCountUnits = Configuration::EnumNewPattern_ClothCountUnits::CM;
      stream >> m_patternTitle;
      stream >> m_patternAuthor;
      stream >> m_patternCopyright;
      stream >> m_patternFabric;
      stream >> m_fabricColor;
      stream >> m_flossScheme;
      stream >> m_patternInstructions;
      break;
    default:
      return false;
  }
  return true;
}
